@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2019 Oracle Corporation
+ * Copyright (C) 2011-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -35,21 +35,10 @@
 #include "VBoxDispDbg.h"
 #include "VBoxDispD3DIf.h"
 #include "../../common/wddm/VBoxMPIf.h"
-#include "VBoxDispCm.h"
 #include "VBoxDispMpInternal.h"
 #include <VBoxDispKmt.h>
-#ifdef VBOX_WITH_CRHGSMI
-#include "VBoxUhgsmiBase.h"
-#include "VBoxUhgsmiDisp.h"
-#include "VBoxUhgsmiKmt.h"
-#endif
 #include "VBoxDispD3D.h"
-#ifndef IN_VBOXCRHGSMI
 #include "VBoxD3DIf.h"
-#endif
-#ifdef VBOX_WITH_CROGL
-#include <cr_protocol.h>
-#endif
 
 # ifdef VBOXWDDMDISP
 #  define VBOXWDDMDISP_DECL(_type) DECLEXPORT(_type)
