@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2019 Oracle Corporation
+ * Copyright (C) 2011-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -37,22 +37,20 @@
 
 /* generic debugging facilities & extra data checks */
 # define VBOXWDDMDISP_DEBUG
-# if defined(DEBUG_misha) || defined(DEBUG_leo)
 /* for some reason when debugging with VirtualKD, user-mode DbgPrint's are discarded
  * the workaround so far is to pass the log info to the kernel driver and DbgPrint'ed from there,
  * which is enabled by this define */
 //#  define VBOXWDDMDISP_DEBUG_PRINTDRV
-/* use OutputDebugString */
-//#  define VBOXWDDMDISP_DEBUG_PRINT
+
+/* Uncomment to use OutputDebugString */
+//#define VBOXWDDMDISP_DEBUG_PRINT
+
 /* disable shared resource creation with wine */
 //#  define VBOXWDDMDISP_DEBUG_NOSHARED
 
 //#  define VBOXWDDMDISP_DEBUG_PRINT_SHARED_CREATE
-
 //#  define VBOXWDDMDISP_DEBUG_TIMER
-# endif
 
-# ifndef IN_VBOXCRHGSMI
 /* debug config vars */
 extern DWORD g_VBoxVDbgFDumpSetTexture;
 extern DWORD g_VBoxVDbgFDumpDrawPrim;
@@ -83,12 +81,9 @@ extern DWORD g_VBoxVDbgCfgForceDummyDevCreate;
 extern struct VBOXWDDMDISP_DEVICE *g_VBoxVDbgInternalDevice;
 extern struct VBOXWDDMDISP_RESOURCE *g_VBoxVDbgInternalRc;
 
-extern DWORD g_VBoxVDbgCfgCreateSwapchainOnDdiOnce;
-
-# endif /* #ifndef IN_VBOXCRHGSMI */
 #endif
 
-#if defined(VBOXWDDMDISP_DEBUG) || defined(VBOX_WDDMDISP_WITH_PROFILE)
+#if defined(VBOXWDDMDISP_DEBUG)
 /* log enable flags */
 extern DWORD g_VBoxVDbgFLogRel;
 extern DWORD g_VBoxVDbgFLog;
@@ -120,7 +115,7 @@ void vboxVDbgVEHandlerUnregister();
 # define DbgPrintUsrFlow(_m) do { } while (0)
 #endif
 
-#if defined(VBOXWDDMDISP_DEBUG) || defined(VBOX_WDDMDISP_WITH_PROFILE)
+#if defined(VBOXWDDMDISP_DEBUG)
 #define vboxVDbgInternalLog(_p) if (g_VBoxVDbgFLog) { _p }
 #define vboxVDbgInternalLogFlow(_p) if (g_VBoxVDbgFLogFlow) { _p }
 #define vboxVDbgInternalLogRel(_p) if (g_VBoxVDbgFLogRel) { _p }
@@ -159,7 +154,7 @@ void vboxVDbgVEHandlerUnregister();
         ); \
     } while (0)
 
-#if defined(VBOXWDDMDISP_DEBUG) || defined(VBOX_WDDMDISP_WITH_PROFILE)
+#if defined(VBOXWDDMDISP_DEBUG)
 extern DWORD g_VBoxVDbgPid;
 extern LONG g_VBoxVDbgFIsDwm;
 #define VBOXVDBG_CHECK_EXE(_pszName) (vboxVDbgDoCheckExe(_pszName))
@@ -203,7 +198,6 @@ void vboxDispLogDrvF(char * szString, ...);
 
 void vboxDispLogDbgPrintF(char * szString, ...);
 
-# ifndef IN_VBOXCRHGSMI
 typedef struct VBOXWDDMDISP_ALLOCATION *PVBOXWDDMDISP_ALLOCATION;
 typedef struct VBOXWDDMDISP_RESOURCE *PVBOXWDDMDISP_RESOURCE;
 
@@ -240,8 +234,6 @@ VOID vboxVDbgDoDumpAllocRect(const char * pPrefix, PVBOXWDDMDISP_ALLOCATION pAll
 VOID vboxVDbgDoDumpRcRect(const char * pPrefix, PVBOXWDDMDISP_ALLOCATION pAlloc, IDirect3DResource9 *pD3DRc, RECT *pRect, const char * pSuffix, DWORD fFlags);
 VOID vboxVDbgDoDumpLockUnlockSurfTex(const char * pPrefix, const VBOXWDDMDISP_ALLOCATION *pAlloc, const char * pSuffix, DWORD fFlags);
 VOID vboxVDbgDoDumpRt(const char * pPrefix, struct VBOXWDDMDISP_DEVICE *pDevice, const char * pSuffix, DWORD fFlags);
-VOID vboxVDbgDoDumpBb(const char * pPrefix, IDirect3DSwapChain9 *pSwapchainIf, const char * pSuffix, DWORD fFlags);
-VOID vboxVDbgDoDumpFb(const char * pPrefix, IDirect3DSwapChain9 *pSwapchainIf, const char * pSuffix, DWORD fFlags);
 VOID vboxVDbgDoDumpSamplers(const char * pPrefix, struct VBOXWDDMDISP_DEVICE *pDevice, const char * pSuffix, DWORD fFlags);
 
 void vboxVDbgDoPrintRect(const char * pPrefix, const RECT *pRect, const char * pSuffix);
@@ -486,12 +478,6 @@ HRESULT vboxVDbgTimerStop(HANDLE hTimerQueue, HANDLE hTimer);
 #define VBOXVDBG_DUMP_BLT_LEAVE(_pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect) \
         VBOXVDBG_DUMP_STRETCH_RECT(Blt, "<==", _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect)
 
-#define VBOXVDBG_DUMP_SWAPCHAIN_SYNC_ENTER(_pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect) \
-        VBOXVDBG_DUMP_STRETCH_RECT(ScSync, "==>", _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect)
-
-#define VBOXVDBG_DUMP_SWAPCHAIN_SYNC_LEAVE(_pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect) \
-        VBOXVDBG_DUMP_STRETCH_RECT(ScSync, "<==", _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect)
-
 #define VBOXVDBG_IS_SKIP_DWM_WND_UPDATE(_pSrcRc, _pSrcRect, _pDstRc, _pDstPoint) ( \
             g_VBoxVDbgFSkipCheckTexBltDwmWndUpdate \
             && ( \
@@ -558,9 +544,6 @@ HRESULT vboxVDbgTimerStop(HANDLE hTimerQueue, HANDLE hTimer);
 #define VBOXVDBG_CHECK_BLT(_opBlt, _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect) \
         VBOXVDBG_CHECK_STRETCH_RECT(Blt, _opBlt, _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect)
 
-#define VBOXVDBG_CHECK_SWAPCHAIN_SYNC(_op, _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect) \
-        VBOXVDBG_CHECK_STRETCH_RECT(ScSync, _op, _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect)
-
 #define VBOXVDBG_DUMP_SYNC_RT(_pBbSurf) do { \
         if (VBOXVDBG_IS_DUMP_ALLOWED(RtSynch)) \
         { \
@@ -568,33 +551,6 @@ HRESULT vboxVDbgTimerStop(HANDLE hTimerQueue, HANDLE hTimer);
         } \
     } while (0)
 
-#define VBOXVDBG_DUMP_PRESENT_ENTER(_pDevice, _pSwapchain) do { \
-        if (VBOXVDBG_IS_DUMP_ALLOWED(PresentEnter)) { \
-            if (!(_pSwapchain)->fFlags.bRtReportingPresent) { \
-                vboxVDbgDoDumpBb("==>"__FUNCTION__" Bb:\n", (_pSwapchain)->pSwapChainIf, "", VBOXVDBG_DUMP_FLAGS_FOR_TYPE(PresentEnter)); \
-            } \
-            else  { \
-                PVBOXWDDMDISP_ALLOCATION pCurBb = vboxWddmSwapchainGetBb((_pSwapchain))->pAlloc; \
-                IDirect3DSurface9 *pSurf; \
-                HRESULT hr = vboxWddmSwapchainSurfGet(_pDevice, _pSwapchain, pCurBb, &pSurf); \
-                Assert(hr == S_OK); \
-                vboxVDbgDoDumpRcRect("== "__FUNCTION__" Bb:\n", pCurBb, pSurf, NULL, "", VBOXVDBG_DUMP_FLAGS_FOR_TYPE(PresentEnter)); \
-                pSurf->Release(); \
-            } \
-        } \
-    } while (0)
-
-#define VBOXVDBG_DUMP_PRESENT_LEAVE(_pDevice, _pSwapchain) do { \
-        if (VBOXVDBG_IS_DUMP_ALLOWED(PresentLeave)) { \
-            if (!(_pSwapchain)->fFlags.bRtReportingPresent) { \
-                vboxVDbgDoDumpFb("<=="__FUNCTION__" Fb:\n", (_pSwapchain)->pSwapChainIf, "", VBOXVDBG_DUMP_FLAGS_FOR_TYPE(PresentLeave)); \
-            } \
-            else  { \
-                vboxVDbgPrint(("PRESENT_LEAVE: unsupported for Rt Reporting mode\n")); \
-            } \
-        } \
-    } while (0)
-
 
 #define VBOXVDBG_DUMP_FLUSH(_pDevice) do { \
         if (VBOXVDBG_IS_DUMP_ALLOWED(Flush)) \
@@ -621,18 +577,6 @@ HRESULT vboxVDbgTimerStop(HANDLE hTimerQueue, HANDLE hTimer);
         } \
     } while (0)
 
-
-#define VBOXVDBG_CREATE_CHECK_SWAPCHAIN() do { \
-            if (g_VBoxVDbgCfgCreateSwapchainOnDdiOnce && g_VBoxVDbgInternalRc) { \
-                PVBOXWDDMDISP_SWAPCHAIN pSwapchain; \
-                HRESULT hr = vboxWddmSwapchainCreateIfForRc(g_VBoxVDbgInternalDevice, g_VBoxVDbgInternalRc, &pSwapchain); \
-                Assert(hr == S_OK); \
-                g_VBoxVDbgInternalRc = NULL; \
-                g_VBoxVDbgCfgCreateSwapchainOnDdiOnce = 0; \
-            } \
-        } while (0)
-
-# endif /* # ifndef IN_VBOXCRHGSMI */
 #else
 #define VBOXVDBG_DUMP_DRAWPRIM_ENTER(_pDevice) do { } while (0)
 #define VBOXVDBG_DUMP_DRAWPRIM_LEAVE(_pDevice) do { } while (0)
@@ -645,8 +589,6 @@ HRESULT vboxVDbgTimerStop(HANDLE hTimerQueue, HANDLE hTimer);
 #define VBOXVDBG_DUMP_FLUSH(_pDevice) do { } while (0)
 #define VBOXVDBG_DUMP_LOCK_ST(_pData) do { } while (0)
 #define VBOXVDBG_DUMP_UNLOCK_ST(_pData) do { } while (0)
-#define VBOXVDBG_DUMP_PRESENT_ENTER(_pDevice, _pSwapchain) do { } while (0)
-#define VBOXVDBG_DUMP_PRESENT_LEAVE(_pDevice, _pSwapchain) do { } while (0)
 #define VBOXVDBG_BREAK_SHARED(_pRc) do { } while (0)
 #define VBOXVDBG_BREAK_SHARED_DEV(_pDevice) do { } while (0)
 #define VBOXVDBG_BREAK_DDI() do { } while (0)
@@ -654,8 +596,6 @@ HRESULT vboxVDbgTimerStop(HANDLE hTimerQueue, HANDLE hTimer);
 #define VBOXVDBG_CHECK_BLT(_opBlt, _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect) do { _opBlt; } while (0)
 #define VBOXVDBG_CHECK_TEXBLT(_opTexBlt, _pSrcRc, _pSrcRect, _pDstRc, _pDstPoint) do { _opTexBlt; } while (0)
 #define VBOXVDBG_ASSERT_IS_DWM(_bDwm) do { } while (0)
-#define VBOXVDBG_CHECK_SWAPCHAIN_SYNC(_op, _pSrcAlloc, _pSrcSurf, _pSrcRect, _pDstAlloc, _pDstSurf, _pDstRect) do { _op; } while (0)
-#define VBOXVDBG_CREATE_CHECK_SWAPCHAIN() do { } while (0)
 #endif
 
 
