@@ -26,24 +26,47 @@ from The Open Group.
 
 */
 
-extern _X_EXPORT void miChangeGC(GCPtr          pGC,
-                                 unsigned long  mask);
 
-extern _X_EXPORT void miDestroyGC(GCPtr         pGC);
+extern void miChangeGC(
+    GCPtr  /*pGC*/,
+    unsigned long /*mask*/
+);
 
-extern _X_EXPORT void miDestroyClip(GCPtr       pGC);
+extern void miDestroyGC(
+    GCPtr  /*pGC*/
+);
 
-extern _X_EXPORT void miChangeClip(GCPtr        pGC,
-                                   int          type,
-                                   void         *pvalue,
-                                   int          nrects);
+extern GCOpsPtr miCreateGCOps(
+    GCOpsPtr /*prototype*/
+);
 
-extern _X_EXPORT void miCopyClip(GCPtr          pgcDst,
-                                 GCPtr          pgcSrc);
+extern void miDestroyGCOps(
+    GCOpsPtr /*ops*/
+);
 
-extern _X_EXPORT void miCopyGC(GCPtr            pGCSrc,
-                               unsigned long    changes,
-                               GCPtr            pGCDst);
+extern void miDestroyClip(
+    GCPtr /*pGC*/
+);
 
-extern _X_EXPORT void miComputeCompositeClip(GCPtr              pGC,
-                                             DrawablePtr        pDrawable);
+extern void miChangeClip(
+    GCPtr   /*pGC*/,
+    int     /*type*/,
+    pointer /*pvalue*/,
+    int     /*nrects*/
+);
+
+extern void miCopyClip(
+    GCPtr /*pgcDst*/,
+    GCPtr /*pgcSrc*/
+);
+
+extern void miCopyGC(
+    GCPtr /*pGCSrc*/,
+    unsigned long /*changes*/,
+    GCPtr /*pGCDst*/
+);
+
+extern void miComputeCompositeClip(
+    GCPtr       /*pGC*/,
+    DrawablePtr /*pDrawable*/
+);
