@@ -8,7 +8,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -34,9 +34,9 @@
 
 #include <VBox/vd.h>
 #include <VBox/sup.h>
-#include <VBox/err.h>
 #include <VBox/log.h>
 
+#include <iprt/ctype.h>
 #include <iprt/file.h>
 #include <iprt/getopt.h>
 #include <iprt/stream.h>
@@ -48,7 +48,7 @@
 
 /* Includes for the raw disk stuff. */
 #ifdef RT_OS_WINDOWS
-# include <windows.h>
+# include <iprt/win/windows.h>
 # include <winioctl.h>
 #elif defined(RT_OS_LINUX) || defined(RT_OS_DARWIN) \
     || defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD)
@@ -123,7 +123,7 @@ typedef struct HOSTPARTITION
 typedef struct HOSTPARTITIONS
 {
     /** partitioning type - MBR or GPT */
-    VBOXHDDPARTTYPE uPartitioningType;
+    VDISKPARTTYPE uPartitioningType;
     unsigned        cPartitions;
     HOSTPARTITION   aPartitions[HOSTPARTITION_MAX];
 } HOSTPARTITIONS, *PHOSTPARTITIONS;
@@ -134,8 +134,11 @@ bool g_fInternalMode;
 /**
  * Print the usage info.
  */
-void printUsageInternal(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
+void printUsageInternal(USAGECATEGORY enmCommand, PRTSTREAM pStrm)
 {
+    Assert(enmCommand != USAGE_INVALID);
+    Assert(enmCommand != USAGE_S_NEWCMD);
+    Assert(enmCommand != USAGE_S_DUMPOPTS);
     RTStrmPrintf(pStrm,
         "Usage: VBoxManage internalcommands <command> [command arguments]\n"
         "\n"
@@ -146,40 +149,40 @@ void printUsageInternal(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
         "         problems. It is completely unsupported and will change in\n"
         "         incompatible ways without warning.\n",
 
-        (u64Cmd & USAGE_LOADMAP)
+        (enmCommand == USAGE_I_LOADMAP || enmCommand == USAGE_S_ALL)
         ? "  loadmap <vmname|uuid> <symfile> <address> [module] [subtrahend] [segment]\n"
           "      This will instruct DBGF to load the given map file\n"
           "      during initialization.  (See also loadmap in the debugger.)\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_LOADSYMS)
+        (enmCommand == USAGE_I_LOADSYMS || enmCommand == USAGE_S_ALL)
         ? "  loadsyms <vmname|uuid> <symfile> [delta] [module] [module address]\n"
           "      This will instruct DBGF to load the given symbol file\n"
           "      during initialization.\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_SETHDUUID)
+        (enmCommand == USAGE_I_SETHDUUID || enmCommand == USAGE_S_ALL)
         ? "  sethduuid <filepath> [<uuid>]\n"
           "       Assigns a new UUID to the given image file. This way, multiple copies\n"
           "       of a container can be registered.\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_SETHDPARENTUUID)
+        (enmCommand == USAGE_I_SETHDPARENTUUID || enmCommand == USAGE_S_ALL)
         ? "  sethdparentuuid <filepath> <uuid>\n"
           "       Assigns a new parent UUID to the given image file.\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_DUMPHDINFO)
+        (enmCommand == USAGE_I_DUMPHDINFO || enmCommand == USAGE_S_ALL)
         ?  "  dumphdinfo <filepath>\n"
            "       Prints information about the image at the given location.\n"
            "\n"
         : "",
-        (u64Cmd & USAGE_LISTPARTITIONS)
+        (enmCommand == USAGE_I_LISTPARTITIONS || enmCommand == USAGE_S_ALL)
         ? "  listpartitions -rawdisk <diskname>\n"
           "       Lists all partitions on <diskname>.\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_CREATERAWVMDK)
+        (enmCommand == USAGE_I_CREATERAWVMDK || enmCommand == USAGE_S_ALL)
         ? "  createrawvmdk -filename <filename> -rawdisk <diskname>\n"
           "                [-partitions <list of partition numbers> [-mbr <filename>] ]\n"
           "                [-relative]\n"
@@ -197,12 +200,12 @@ void printUsageInternal(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
           "         VBoxManage internalcommands listpartitions\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_RENAMEVMDK)
+        (enmCommand == USAGE_I_RENAMEVMDK || enmCommand == USAGE_S_ALL)
         ? "  renamevmdk -from <filename> -to <filename>\n"
           "       Renames an existing VMDK image, including the base file and all its extents.\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_CONVERTTORAW)
+        (enmCommand == USAGE_I_CONVERTTORAW || enmCommand == USAGE_S_ALL)
         ? "  converttoraw [-format <fileformat>] <filename> <outputfile>"
 #ifdef ENABLE_CONVERT_RAW_TO_STDOUT
           "|stdout"
@@ -215,14 +218,14 @@ void printUsageInternal(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
           ".\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_CONVERTHD)
+        (enmCommand == USAGE_I_CONVERTHD || enmCommand == USAGE_S_ALL)
         ? "  converthd [-srcformat VDI|VMDK|VHD|RAW]\n"
           "            [-dstformat VDI|VMDK|VHD|RAW]\n"
           "            <inputfile> <outputfile>\n"
           "       converts hard disk images between formats\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_REPAIRHD)
+        (enmCommand == USAGE_I_REPAIRHD || enmCommand == USAGE_S_ALL)
         ? "  repairhd [-dry-run]\n"
           "           [-format VDI|VMDK|VHD|...]\n"
           "           <filename>\n"
@@ -230,12 +233,12 @@ void printUsageInternal(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
           "\n"
         : "",
 #ifdef RT_OS_WINDOWS
-        (u64Cmd & USAGE_MODINSTALL)
+        (enmCommand == USAGE_I_MODINSTALL || enmCommand == USAGE_S_ALL)
         ? "  modinstall\n"
           "       Installs the necessary driver for the host OS\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_MODUNINSTALL)
+        (enmCommand == USAGE_I_MODUNINSTALL || enmCommand == USAGE_S_ALL)
         ? "  moduninstall\n"
           "       Deinstalls the driver\n"
           "\n"
@@ -244,18 +247,18 @@ void printUsageInternal(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
         "",
         "",
 #endif
-        (u64Cmd & USAGE_DEBUGLOG)
+        (enmCommand == USAGE_I_DEBUGLOG || enmCommand == USAGE_S_ALL)
         ? "  debuglog <vmname|uuid> [--enable|--disable] [--flags todo]\n"
           "           [--groups todo] [--destinations todo]\n"
           "       Controls debug logging.\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_PASSWORDHASH)
-        ? "  passwordhash <passsword>\n"
+        (enmCommand == USAGE_I_PASSWORDHASH || enmCommand == USAGE_S_ALL)
+        ? "  passwordhash <password>\n"
           "       Generates a password hash.\n"
           "\n"
         : "",
-        (u64Cmd & USAGE_GUESTSTATS)
+        (enmCommand == USAGE_I_GUESTSTATS || enmCommand == USAGE_S_ALL)
         ? "  gueststats <vmname|uuid> [--interval <seconds>]\n"
           "       Obtains and prints internal guest statistics.\n"
           "       Sets the update interval if specified.\n"
@@ -451,6 +454,7 @@ static HRESULT SetInt64(ComPtr<IMachine> pMachine, const char *pszKeyBase, const
  */
 static RTEXITCODE CmdLoadSyms(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aSession);
     HRESULT rc;
 
     /*
@@ -466,7 +470,7 @@ static RTEXITCODE CmdLoadSyms(int argc, char **argv, ComPtr<IVirtualBox> aVirtua
     const char *pszFilename;
     int64_t     offDelta = 0;
     const char *pszModule = NULL;
-    uint64_t    ModuleAddress = ~0;
+    uint64_t    ModuleAddress = UINT64_MAX;
     uint64_t    ModuleSize = 0;
 
     /* filename */
@@ -527,6 +531,7 @@ static RTEXITCODE CmdLoadSyms(int argc, char **argv, ComPtr<IVirtualBox> aVirtua
  */
 static RTEXITCODE CmdLoadMap(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aSession);
     HRESULT rc;
 
     /*
@@ -599,6 +604,7 @@ static RTEXITCODE CmdLoadMap(int argc, char **argv, ComPtr<IVirtualBox> aVirtual
 
 static DECLCALLBACK(void) handleVDError(void *pvUser, int rc, RT_SRC_POS_DECL, const char *pszFormat, va_list va)
 {
+    RT_NOREF(pvUser);
     RTMsgErrorV(pszFormat, va);
     RTMsgError("Error code %Rrc at %s(%u) in function %s", rc, RT_SRC_POS_ARGS);
 }
@@ -611,6 +617,7 @@ static DECLCALLBACK(int) handleVDMessage(void *pvUser, const char *pszFormat, va
 
 static RTEXITCODE CmdSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
     Guid uuid;
     RTUUID rtuuid;
     enum eUuidType {
@@ -622,12 +629,12 @@ static RTEXITCODE CmdSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtu
     {
         uuidType = HDUUID;
         if (argc != 3 && argc != 2)
-            return errorSyntax(USAGE_SETHDUUID, "Not enough parameters");
+            return errorSyntax(USAGE_I_SETHDUUID, "Not enough parameters");
         /* if specified, take UUID, otherwise generate a new one */
         if (argc == 3)
         {
             if (RT_FAILURE(RTUuidFromStr(&rtuuid, argv[2])))
-                return errorSyntax(USAGE_SETHDUUID, "Invalid UUID parameter");
+                return errorSyntax(USAGE_I_SETHDUUID, "Invalid UUID parameter");
             uuid = argv[2];
         } else
             uuid.create();
@@ -636,23 +643,23 @@ static RTEXITCODE CmdSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtu
     {
         uuidType = HDPARENTUUID;
         if (argc != 3)
-            return errorSyntax(USAGE_SETHDPARENTUUID, "Not enough parameters");
+            return errorSyntax(USAGE_I_SETHDPARENTUUID, "Not enough parameters");
         if (RT_FAILURE(RTUuidFromStr(&rtuuid, argv[2])))
-            return errorSyntax(USAGE_SETHDPARENTUUID, "Invalid UUID parameter");
+            return errorSyntax(USAGE_I_SETHDPARENTUUID, "Invalid UUID parameter");
         uuid = argv[2];
     }
     else
-        return errorSyntax(USAGE_SETHDUUID, "Invalid invocation");
+        return errorSyntax(USAGE_I_SETHDUUID, "Invalid invocation");
 
     /* just try it */
     char *pszFormat = NULL;
     VDTYPE enmType = VDTYPE_INVALID;
     int rc = VDGetFormat(NULL /* pVDIfsDisk */, NULL /* pVDIfsImage */,
-                         argv[1], &pszFormat, &enmType);
+                         argv[1], VDTYPE_INVALID, &pszFormat, &enmType);
     if (RT_FAILURE(rc))
         return RTMsgErrorExit(RTEXITCODE_FAILURE, "Format autodetect failed: %Rrc", rc);
 
-    PVBOXHDD pDisk = NULL;
+    PVDISK pDisk = NULL;
 
     PVDINTERFACE     pVDIfs = NULL;
     VDINTERFACEERROR vdInterfaceError;
@@ -689,21 +696,23 @@ static RTEXITCODE CmdSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtu
 
 static RTEXITCODE CmdDumpHDInfo(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
+
     /* we need exactly one parameter: the image file */
     if (argc != 1)
     {
-        return errorSyntax(USAGE_DUMPHDINFO, "Not enough parameters");
+        return errorSyntax(USAGE_I_DUMPHDINFO, "Not enough parameters");
     }
 
     /* just try it */
     char *pszFormat = NULL;
     VDTYPE enmType = VDTYPE_INVALID;
     int rc = VDGetFormat(NULL /* pVDIfsDisk */, NULL /* pVDIfsImage */,
-                         argv[0], &pszFormat, &enmType);
+                         argv[0], VDTYPE_INVALID, &pszFormat, &enmType);
     if (RT_FAILURE(rc))
         return RTMsgErrorExit(RTEXITCODE_FAILURE, "Format autodetect failed: %Rrc", rc);
 
-    PVBOXHDD pDisk = NULL;
+    PVDISK pDisk = NULL;
 
     PVDINTERFACE     pVDIfs = NULL;
     VDINTERFACEERROR vdInterfaceError;
@@ -738,7 +747,7 @@ static int partRead(RTFILE File, PHOSTPARTITIONS pPart)
     uint64_t lastUsableLBA = 0;
     int rc;
 
-    VBOXHDDPARTTYPE partitioningType;
+    VDISKPARTTYPE partitioningType;
 
     pPart->cPartitions = 0;
     memset(pPart->aPartitions, '\0', sizeof(pPart->aPartitions));
@@ -767,6 +776,7 @@ static int partRead(RTFILE File, PHOSTPARTITIONS pPart)
 
             /** @todo r=bird: C have this handy concept called structures which
              *        greatly simplify data access...  (Someone is really lazy here!) */
+#if 0 /* unused */
             uint64_t firstUsableLBA     = RT_MAKE_U64_FROM_U8(partitionTableHeader[40],
                                                               partitionTableHeader[41],
                                                               partitionTableHeader[42],
@@ -776,6 +786,7 @@ static int partRead(RTFILE File, PHOSTPARTITIONS pPart)
                                                               partitionTableHeader[46],
                                                               partitionTableHeader[47]
                                                               );
+#endif
             lastUsableLBA               = RT_MAKE_U64_FROM_U8(partitionTableHeader[48],
                                                               partitionTableHeader[49],
                                                               partitionTableHeader[50],
@@ -1067,6 +1078,7 @@ static int partRead(RTFILE File, PHOSTPARTITIONS pPart)
 
 static RTEXITCODE CmdListPartitions(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
     Utf8Str rawdisk;
 
     /* let's have a closer look at the arguments */
@@ -1083,12 +1095,12 @@ static RTEXITCODE CmdListPartitions(int argc, char **argv, ComPtr<IVirtualBox> a
         }
         else
         {
-            return errorSyntax(USAGE_LISTPARTITIONS, "Invalid parameter '%s'", argv[i]);
+            return errorSyntax(USAGE_I_LISTPARTITIONS, "Invalid parameter '%s'", argv[i]);
         }
     }
 
     if (rawdisk.isEmpty())
-        return errorSyntax(USAGE_LISTPARTITIONS, "Mandatory parameter -rawdisk missing");
+        return errorSyntax(USAGE_I_LISTPARTITIONS, "Mandatory parameter -rawdisk missing");
 
     RTFILE hRawFile;
     int vrc = RTFileOpen(&hRawFile, rawdisk.c_str(), RTFILE_O_READ | RTFILE_O_OPEN | RTFILE_O_DENY_WRITE);
@@ -1123,17 +1135,17 @@ static RTEXITCODE CmdListPartitions(int argc, char **argv, ComPtr<IVirtualBox> a
     return RT_SUCCESS(vrc) ? RTEXITCODE_SUCCESS : RTEXITCODE_FAILURE;
 }
 
-static PVBOXHDDRAWPARTDESC appendPartDesc(uint32_t *pcPartDescs, PVBOXHDDRAWPARTDESC *ppPartDescs)
+static PVDISKRAWPARTDESC appendPartDesc(uint32_t *pcPartDescs, PVDISKRAWPARTDESC *ppPartDescs)
 {
     (*pcPartDescs)++;
-    PVBOXHDDRAWPARTDESC p;
-    p = (PVBOXHDDRAWPARTDESC)RTMemRealloc(*ppPartDescs,
-                                          *pcPartDescs * sizeof(VBOXHDDRAWPARTDESC));
+    PVDISKRAWPARTDESC p;
+    p = (PVDISKRAWPARTDESC)RTMemRealloc(*ppPartDescs,
+                                          *pcPartDescs * sizeof(VDISKRAWPARTDESC));
     *ppPartDescs = p;
     if (p)
     {
         p = p + *pcPartDescs - 1;
-        memset(p, '\0', sizeof(VBOXHDDRAWPARTDESC));
+        memset(p, '\0', sizeof(VDISKRAWPARTDESC));
     }
 
     return p;
@@ -1141,6 +1153,7 @@ static PVBOXHDDRAWPARTDESC appendPartDesc(uint32_t *pcPartDescs, PVBOXHDDRAWPART
 
 static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
     HRESULT rc = S_OK;
     Utf8Str filename;
     const char *pszMBRFilename = NULL;
@@ -1149,8 +1162,8 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
     bool fRelative = false;
 
     uint64_t cbSize = 0;
-    PVBOXHDD pDisk = NULL;
-    VBOXHDDRAW RawDescriptor;
+    PVDISK pDisk = NULL;
+    VDISKRAW RawDescriptor;
     PVDINTERFACE pVDIfs = NULL;
 
     /* let's have a closer look at the arguments */
@@ -1199,15 +1212,15 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
         }
 #endif /* RT_OS_LINUX || RT_OS_FREEBSD */
         else
-            return errorSyntax(USAGE_CREATERAWVMDK, "Invalid parameter '%s'", argv[i]);
+            return errorSyntax(USAGE_I_CREATERAWVMDK, "Invalid parameter '%s'", argv[i]);
     }
 
     if (filename.isEmpty())
-        return errorSyntax(USAGE_CREATERAWVMDK, "Mandatory parameter -filename missing");
+        return errorSyntax(USAGE_I_CREATERAWVMDK, "Mandatory parameter -filename missing");
     if (rawdisk.isEmpty())
-        return errorSyntax(USAGE_CREATERAWVMDK, "Mandatory parameter -rawdisk missing");
+        return errorSyntax(USAGE_I_CREATERAWVMDK, "Mandatory parameter -rawdisk missing");
     if (!pszPartitions && pszMBRFilename)
-        return errorSyntax(USAGE_CREATERAWVMDK, "The parameter -mbr is only valid when the parameter -partitions is also present");
+        return errorSyntax(USAGE_I_CREATERAWVMDK, "The parameter -mbr is only valid when the parameter -partitions is also present");
 
 #ifdef RT_OS_DARWIN
     fRelative = true;
@@ -1274,7 +1287,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
          * before failing.
          */
         vrc = RTErrConvertFromWin32(GetLastError());
-        if (RT_FAILURE(RTFileGetSize(hRawFile, &cbSize)))
+        if (RT_FAILURE(RTFileQuerySize(hRawFile, &cbSize)))
         {
             RTMsgError("Cannot get the geometry of the raw disk '%s': %Rrc", rawdisk.c_str(), vrc);
             goto out;
@@ -1324,7 +1337,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
         }
         else if (S_ISREG(DevStat.st_mode))
         {
-            vrc = RTFileGetSize(hRawFile, &cbSize);
+            vrc = RTFileQuerySize(hRawFile, &cbSize);
             if (RT_FAILURE(vrc))
             {
                 RTMsgError("Failed to get size of file '%s': %Rrc", rawdisk.c_str(), vrc);
@@ -1379,7 +1392,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
         else if (S_ISREG(DevStat.st_mode))
         {
             fRelative = false; /* Must be false for raw image files. */
-            vrc = RTFileGetSize(hRawFile, &cbSize);
+            vrc = RTFileQuerySize(hRawFile, &cbSize);
             if (RT_FAILURE(vrc))
             {
                 RTMsgError("Failed to get size of file '%s': %Rrc", rawdisk.c_str(), vrc);
@@ -1417,7 +1430,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
         }
         else if (S_ISREG(DevStat.st_mode))
         {
-            vrc = RTFileGetSize(hRawFile, &cbSize);
+            vrc = RTFileQuerySize(hRawFile, &cbSize);
             if (RT_FAILURE(vrc))
             {
                 RTMsgError("Failed to get size of file '%s': %Rrc", rawdisk.c_str(), vrc);
@@ -1479,7 +1492,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
 #else /* all unrecognized OSes */
     /* Hopefully this works on all other hosts. If it doesn't, it'll just fail
      * creating the VMDK, so no real harm done. */
-    vrc = RTFileGetSize(hRawFile, &cbSize);
+    vrc = RTFileQuerySize(hRawFile, &cbSize);
     if (RT_FAILURE(vrc))
     {
         RTMsgError("Cannot get the size of the raw disk '%s': %Rrc", rawdisk.c_str(), vrc);
@@ -1490,7 +1503,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
     /* Check whether cbSize is actually sensible. */
     if (!cbSize || cbSize % 512)
     {
-        RTMsgError("Detected size of raw disk '%s' is %s, an invalid value", rawdisk.c_str(), cbSize);
+        RTMsgError("Detected size of raw disk '%s' is %RU64, an invalid value", rawdisk.c_str(), cbSize);
         vrc = VERR_INVALID_PARAMETER;
         goto out;
     }
@@ -1501,12 +1514,12 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
     RawDescriptor.szSignature[3] = '\0';
     if (!pszPartitions)
     {
-        RawDescriptor.uFlags = VBOXHDDRAW_DISK;
+        RawDescriptor.uFlags = VDISKRAW_DISK;
         RawDescriptor.pszRawDisk = rawdisk.c_str();
     }
     else
     {
-        RawDescriptor.uFlags = VBOXHDDRAW_NORMAL;
+        RawDescriptor.uFlags = VDISKRAW_NORMAL;
         RawDescriptor.pszRawDisk = NULL;
         RawDescriptor.cPartDescs = 0;
         RawDescriptor.pPartDescs = NULL;
@@ -1571,7 +1584,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
 
         for (unsigned i = 0; i < partitions.cPartitions; i++)
         {
-            PVBOXHDDRAWPARTDESC pPartDesc = NULL;
+            PVDISKRAWPARTDESC pPartDesc = NULL;
 
             /* first dump the MBR/EPT data area */
             if (partitions.aPartitions[i].cPartDataSectors)
@@ -1653,21 +1666,38 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
             if (uPartitions & RT_BIT(partitions.aPartitions[i].uIndex))
             {
                 if (uPartitionsRO & RT_BIT(partitions.aPartitions[i].uIndex))
-                    pPartDesc->uFlags |= VBOXHDDRAW_READONLY;
+                    pPartDesc->uFlags |= VDISKRAW_READONLY;
 
                 if (fRelative)
                 {
 #if defined(RT_OS_LINUX) || defined(RT_OS_DARWIN) || defined(RT_OS_FREEBSD)
                     /* Refer to the correct partition and use offset 0. */
                     char *psz;
-                    RTStrAPrintf(&psz,
 #if defined(RT_OS_LINUX)
-                                 "%s%u",
+                    /*
+                     * Check whether raw disk ends with a digit. In that case
+                     * insert a p before adding the partition number.
+                     * This is used for nvme devices only currently which look like
+                     * /dev/nvme0n1p1 but might be extended to other devices in the
+                     * future.
+                     */
+                    size_t cchRawDisk = rawdisk.length();
+                    if (RT_C_IS_DIGIT(pszRawName[cchRawDisk - 1]))
+                        RTStrAPrintf(&psz,
+                                     "%sp%u",
+                                     rawdisk.c_str(),
+                                     partitions.aPartitions[i].uIndex);
+                    else
+                        RTStrAPrintf(&psz,
+                                     "%s%u",
+                                     rawdisk.c_str(),
+                                     partitions.aPartitions[i].uIndex);
 #elif defined(RT_OS_DARWIN) || defined(RT_OS_FREEBSD)
+                    RTStrAPrintf(&psz,
                                  "%ss%u",
-#endif
                                  rawdisk.c_str(),
                                  partitions.aPartitions[i].uIndex);
+#endif
                     if (!psz)
                     {
                         vrc = VERR_NO_STR_MEMORY;
@@ -1727,7 +1757,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
             if (uMinIdx != i)
             {
                 /* Swap entries at index i and uMinIdx. */
-                VBOXHDDRAWPARTDESC tmp;
+                VDISKRAWPARTDESC tmp;
                 memcpy(&tmp, &RawDescriptor.pPartDescs[i], sizeof(tmp));
                 memcpy(&RawDescriptor.pPartDescs[i], &RawDescriptor.pPartDescs[uMinIdx], sizeof(tmp));
                 memcpy(&RawDescriptor.pPartDescs[uMinIdx], &tmp, sizeof(tmp));
@@ -1767,7 +1797,7 @@ static RTEXITCODE CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aV
     RTFileClose(hRawFile);
 
 #ifdef DEBUG_klaus
-    if (!(RawDescriptor.uFlags & VBOXHDDRAW_DISK))
+    if (!(RawDescriptor.uFlags & VDISKRAW_DISK))
     {
         RTPrintf("#            start         length    startoffset  partdataptr  device\n");
         for (unsigned i = 0; i < RawDescriptor.cPartDescs; i++)
@@ -1843,6 +1873,7 @@ out:
 
 static RTEXITCODE CmdRenameVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
     Utf8Str src;
     Utf8Str dst;
     /* Parse the arguments. */
@@ -1868,16 +1899,16 @@ static RTEXITCODE CmdRenameVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirt
         }
         else
         {
-            return errorSyntax(USAGE_RENAMEVMDK, "Invalid parameter '%s'", argv[i]);
+            return errorSyntax(USAGE_I_RENAMEVMDK, "Invalid parameter '%s'", argv[i]);
         }
     }
 
     if (src.isEmpty())
-        return errorSyntax(USAGE_RENAMEVMDK, "Mandatory parameter -from missing");
+        return errorSyntax(USAGE_I_RENAMEVMDK, "Mandatory parameter -from missing");
     if (dst.isEmpty())
-        return errorSyntax(USAGE_RENAMEVMDK, "Mandatory parameter -to missing");
+        return errorSyntax(USAGE_I_RENAMEVMDK, "Mandatory parameter -to missing");
 
-    PVBOXHDD pDisk = NULL;
+    PVDISK pDisk = NULL;
 
     PVDINTERFACE     pVDIfs = NULL;
     VDINTERFACEERROR vdInterfaceError;
@@ -1909,6 +1940,7 @@ static RTEXITCODE CmdRenameVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirt
 
 static RTEXITCODE CmdConvertToRaw(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
     Utf8Str srcformat;
     Utf8Str src;
     Utf8Str dst;
@@ -1940,16 +1972,16 @@ static RTEXITCODE CmdConvertToRaw(int argc, char **argv, ComPtr<IVirtualBox> aVi
         }
         else
         {
-            return errorSyntax(USAGE_CONVERTTORAW, "Invalid parameter '%s'", argv[i]);
+            return errorSyntax(USAGE_I_CONVERTTORAW, "Invalid parameter '%s'", argv[i]);
         }
     }
 
     if (src.isEmpty())
-        return errorSyntax(USAGE_CONVERTTORAW, "Mandatory filename parameter missing");
+        return errorSyntax(USAGE_I_CONVERTTORAW, "Mandatory filename parameter missing");
     if (dst.isEmpty())
-        return errorSyntax(USAGE_CONVERTTORAW, "Mandatory outputfile parameter missing");
+        return errorSyntax(USAGE_I_CONVERTTORAW, "Mandatory outputfile parameter missing");
 
-    PVBOXHDD pDisk = NULL;
+    PVDISK pDisk = NULL;
 
     PVDINTERFACE     pVDIfs = NULL;
     VDINTERFACEERROR vdInterfaceError;
@@ -1960,7 +1992,7 @@ static RTEXITCODE CmdConvertToRaw(int argc, char **argv, ComPtr<IVirtualBox> aVi
                              NULL, sizeof(VDINTERFACEERROR), &pVDIfs);
     AssertRC(vrc);
 
-    /** @todo: Support convert to raw for floppy and DVD images too. */
+    /** @todo Support convert to raw for floppy and DVD images too. */
     vrc = VDCreate(pVDIfs, VDTYPE_HDD, &pDisk);
     if (RT_FAILURE(vrc))
         return RTMsgErrorExit(RTEXITCODE_FAILURE, "Cannot create the virtual disk container: %Rrc", vrc);
@@ -1983,7 +2015,7 @@ static RTEXITCODE CmdConvertToRaw(int argc, char **argv, ComPtr<IVirtualBox> aVi
         char *pszFormat = NULL;
         VDTYPE enmType = VDTYPE_INVALID;
         vrc = VDGetFormat(NULL /* pVDIfsDisk */, NULL /* pVDIfsImage */,
-                          src.c_str(), &pszFormat, &enmType);
+                          src.c_str(), VDTYPE_INVALID, &pszFormat, &enmType);
         if (RT_FAILURE(vrc) || enmType != VDTYPE_HDD)
         {
             VDCloseAll(pDisk);
@@ -2064,13 +2096,14 @@ static RTEXITCODE CmdConvertToRaw(int argc, char **argv, ComPtr<IVirtualBox> aVi
 
 static RTEXITCODE CmdConvertHardDisk(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
     Utf8Str srcformat;
     Utf8Str dstformat;
     Utf8Str src;
     Utf8Str dst;
     int vrc;
-    PVBOXHDD pSrcDisk = NULL;
-    PVBOXHDD pDstDisk = NULL;
+    PVDISK pSrcDisk = NULL;
+    PVDISK pDstDisk = NULL;
     VDTYPE enmSrcType = VDTYPE_INVALID;
 
     /* Parse the arguments. */
@@ -2104,14 +2137,14 @@ static RTEXITCODE CmdConvertHardDisk(int argc, char **argv, ComPtr<IVirtualBox>
         }
         else
         {
-            return errorSyntax(USAGE_CONVERTHD, "Invalid parameter '%s'", argv[i]);
+            return errorSyntax(USAGE_I_CONVERTHD, "Invalid parameter '%s'", argv[i]);
         }
     }
 
     if (src.isEmpty())
-        return errorSyntax(USAGE_CONVERTHD, "Mandatory input image parameter missing");
+        return errorSyntax(USAGE_I_CONVERTHD, "Mandatory input image parameter missing");
     if (dst.isEmpty())
-        return errorSyntax(USAGE_CONVERTHD, "Mandatory output image parameter missing");
+        return errorSyntax(USAGE_I_CONVERTHD, "Mandatory output image parameter missing");
 
 
     PVDINTERFACE     pVDIfs = NULL;
@@ -2130,7 +2163,7 @@ static RTEXITCODE CmdConvertHardDisk(int argc, char **argv, ComPtr<IVirtualBox>
         {
             char *pszFormat = NULL;
             vrc = VDGetFormat(NULL /* pVDIfsDisk */, NULL /* pVDIfsImage */,
-                              src.c_str(), &pszFormat, &enmSrcType);
+                              src.c_str(), VDTYPE_HDD, &pszFormat, &enmSrcType);
             if (RT_FAILURE(vrc))
             {
                 RTMsgError("No file format specified and autodetect failed - please specify format: %Rrc", vrc);
@@ -2195,11 +2228,11 @@ static RTEXITCODE CmdConvertHardDisk(int argc, char **argv, ComPtr<IVirtualBox>
  */
 static RTEXITCODE CmdRepairHardDisk(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
     Utf8Str image;
     Utf8Str format;
     int vrc;
     bool fDryRun = false;
-    PVBOXHDD pDisk = NULL;
 
     /* Parse the arguments. */
     for (int i = 0; i < argc; i++)
@@ -2223,12 +2256,12 @@ static RTEXITCODE CmdRepairHardDisk(int argc, char **argv, ComPtr<IVirtualBox> a
         }
         else
         {
-            return errorSyntax(USAGE_REPAIRHD, "Invalid parameter '%s'", argv[i]);
+            return errorSyntax(USAGE_I_REPAIRHD, "Invalid parameter '%s'", argv[i]);
         }
     }
 
     if (image.isEmpty())
-        return errorSyntax(USAGE_REPAIRHD, "Mandatory input image parameter missing");
+        return errorSyntax(USAGE_I_REPAIRHD, "Mandatory input image parameter missing");
 
     PVDINTERFACE     pVDIfs = NULL;
     VDINTERFACEERROR vdInterfaceError;
@@ -2248,7 +2281,7 @@ static RTEXITCODE CmdRepairHardDisk(int argc, char **argv, ComPtr<IVirtualBox> a
             VDTYPE enmSrcType = VDTYPE_INVALID;
 
             vrc = VDGetFormat(NULL /* pVDIfsDisk */, NULL /* pVDIfsImage */,
-                              image.c_str(), &pszFormat, &enmSrcType);
+                              image.c_str(), VDTYPE_HDD, &pszFormat, &enmSrcType);
             if (RT_FAILURE(vrc) && (vrc != VERR_VD_IMAGE_CORRUPTED))
             {
                 RTMsgError("No file format specified and autodetect failed - please specify format: %Rrc", vrc);
@@ -2302,7 +2335,7 @@ static RTEXITCODE CmdDebugLog(int argc, char **argv, ComPtr<IVirtualBox> aVirtua
      * that we wish to open.
      */
     if (argc < 1)
-        return errorSyntax(USAGE_DEBUGLOG, "Missing VM name/UUID");
+        return errorSyntax(USAGE_I_DEBUGLOG, "Missing VM name/UUID");
 
     ComPtr<IMachine> ptrMachine;
     HRESULT rc;
@@ -2390,7 +2423,7 @@ static RTEXITCODE CmdDebugLog(int argc, char **argv, ComPtr<IVirtualBox> aVirtua
                 break;
 
             default:
-                return errorGetOpt(USAGE_DEBUGLOG, ch, &ValueUnion);
+                return errorGetOpt(USAGE_I_DEBUGLOG, ch, &ValueUnion);
         }
     }
 
@@ -2414,9 +2447,11 @@ static RTEXITCODE CmdDebugLog(int argc, char **argv, ComPtr<IVirtualBox> aVirtua
  */
 static RTEXITCODE CmdGeneratePasswordHash(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
+    RT_NOREF(aVirtualBox, aSession);
+
     /* one parameter, the password to hash */
     if (argc != 1)
-        return errorSyntax(USAGE_PASSWORDHASH, "password to hash required");
+        return errorSyntax(USAGE_I_PASSWORDHASH, "password to hash required");
 
     uint8_t abDigest[RTSHA256_HASH_SIZE];
     RTSha256(argv[0], strlen(argv[0]), abDigest);
@@ -2435,7 +2470,7 @@ static RTEXITCODE CmdGuestStats(int argc, char **argv, ComPtr<IVirtualBox> aVirt
 {
     /* one parameter, guest name */
     if (argc < 1)
-        return errorSyntax(USAGE_GUESTSTATS, "Missing VM name/UUID");
+        return errorSyntax(USAGE_I_GUESTSTATS, "Missing VM name/UUID");
 
     /*
      * Parse the command.
@@ -2460,12 +2495,12 @@ static RTEXITCODE CmdGuestStats(int argc, char **argv, ComPtr<IVirtualBox> aVirt
                 break;
 
             default:
-                return errorGetOpt(USAGE_GUESTSTATS, ch, &ValueUnion);
+                return errorGetOpt(USAGE_I_GUESTSTATS, ch, &ValueUnion);
         }
     }
 
     if (argc > 1 && aUpdateInterval == 0)
-        return errorSyntax(USAGE_GUESTSTATS, "Invalid update interval specified");
+        return errorSyntax(USAGE_I_GUESTSTATS, "Invalid update interval specified");
 
     RTPrintf("argc=%d interval=%u\n", argc, aUpdateInterval);
 
@@ -2520,7 +2555,7 @@ RTEXITCODE handleInternalCommands(HandlerArg *a)
 
     /* at least a command is required */
     if (a->argc < 1)
-        return errorSyntax(USAGE_ALL, "Command missing");
+        return errorSyntax(USAGE_S_ALL, "Command missing");
 
     /*
      * The 'string switch' on command name.
@@ -2560,6 +2595,6 @@ RTEXITCODE handleInternalCommands(HandlerArg *a)
         return CmdRepairHardDisk(a->argc - 1, &a->argv[1], a->virtualBox, a->session);
 
     /* default: */
-    return errorSyntax(USAGE_ALL, "Invalid command '%s'", a->argv[0]);
+    return errorSyntax(USAGE_S_ALL, "Invalid command '%s'", a->argv[0]);
 }
 
