@@ -1,10 +1,10 @@
 /* $Id: ProgressProxyImpl.h $ */
 /** @file
- * IProgress implementation for Machine::openRemoteSession in VBoxSVC.
+ * IProgress implementation for Machine::LaunchVMProcess in VBoxSVC.
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ____H_PROGRESSPROXYIMPL
-#define ____H_PROGRESSPROXYIMPL
+#ifndef MAIN_INCLUDED_ProgressProxyImpl_h
+#define MAIN_INCLUDED_ProgressProxyImpl_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include "ProgressImpl.h"
 #include "AutoCaller.h"
@@ -36,27 +39,30 @@ public:
     DECLARE_PROTECT_FINAL_CONSTRUCT()
 
     BEGIN_COM_MAP(ProgressProxy)
-        VBOX_DEFAULT_INTERFACE_ENTRIES (IProgress)
+        COM_INTERFACE_ENTRY(ISupportErrorInfo)
+        COM_INTERFACE_ENTRY(IProgress)
+        COM_INTERFACE_ENTRY2(IDispatch, IProgress)
+        VBOX_TWEAK_INTERFACE_ENTRY(IProgress)
     END_COM_MAP()
 
     HRESULT FinalConstruct();
     void    FinalRelease();
     HRESULT init(
-#if !defined (VBOX_COM_INPROC)
+#ifndef VBOX_COM_INPROC
                  VirtualBox *pParent,
 #endif
                  IUnknown *pInitiator,
-                 CBSTR bstrDescription,
+                 Utf8Str strDescription,
                  BOOL fCancelable);
     HRESULT init(
-#if !defined (VBOX_COM_INPROC)
+#ifndef VBOX_COM_INPROC
                  VirtualBox *pParent,
 #endif
                  IUnknown *pInitiator,
-                 CBSTR bstrDescription,
+                 Utf8Str strDescription,
                  BOOL fCancelable,
                  ULONG uTotalOperationsWeight,
-                 CBSTR bstrFirstOperationDescription,
+                 Utf8Str strFirstOperationDescription,
                  ULONG uFirstOperationWeight,
                  ULONG cOtherProgressObjectOperations);
     void    uninit();
@@ -110,5 +116,5 @@ private:
 
 };
 
-#endif /* !____H_PROGRESSPROXYIMPL */
+#endif /* !MAIN_INCLUDED_ProgressProxyImpl_h */
 
