@@ -33,6 +33,38 @@ fi
 MY_SCRIPT_NAME="backport-commit.sh"
 . "${MY_SCRIPT_DIR}/backport-common.sh"
 
+#
+# If no revisions was given, try figure it out from the svn:merge-info
+# property.
+#
+if test -z "${MY_REVISIONS}"; then
+    MY_REV_TMP=backport-revisions.tmp
+    if ! svn di --properties-only --depth empty "${MY_BRANCH_DIR}" > "${MY_REV_TMP}"; then
+        echo "error: failed to get revisions from svn:mergeinfo (svn)"
+        exit 1;
+    fi
+    for MY_REV in $("${MY_SED}" -e '/ *Merged \//!d' -e "s/^ [^:]*:[r]*//" -e 's/,[r]*/ /g' "${MY_REV_TMP}");
+    do
+        case "${MY_REV}" in
+            [0-9][0-9][0-9][0-9][0-9]|[0-9][0-9][0-9][0-9][0-9][0-9]|[0-9][0-9][0-9][0-9][0-9][0-9][0-9])
+                AddRevision "${MY_REV}"
+                ;;
+            [0-9][0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9][0-9]|[0-9][0-9][0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9][0-9][0-9]|[0-9][0-9][0-9][0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9][0-9][0-9][0-9])
+                AddRevisionRange "${MY_REV}"
+                ;;
+
+            *)  echo "error: failed to get revisions from svn:mergeinfo - does not grok: ${MY_ARG}"
+                exit 1;;
+        esac
+    done
+    "${MY_RM}" -f -- "${MY_REV_TMP}"
+    if test -z "${MY_REVISIONS}"; then
+        echo "error: No backported revisions found";
+        exit 1;
+    fi
+    echo "info: Detected revisions: ${MY_REVISIONS}"
+fi
+
 #
 # Generate the commit message into MY_MSG_FILE.
 #
@@ -66,12 +98,13 @@ else
         MY_COUNTER=`"${MY_EXPR}" ${MY_COUNTER} + 1`
     done
     echo "." >> "${MY_MSG_FILE}"
+    echo ""  >> "${MY_MSG_FILE}"
 
     # One bullet with the commit text.
     for MY_REV in ${MY_REVISIONS};
     do
         echo -n "* r${MY_REV}: " >> "${MY_MSG_FILE}"
-        if ! "${MY_SVN}" log "-r${MY_REV}" "${MY_TRUNK}" > "${MY_TMP_FILE}"; then
+        if ! "${MY_SVN}" log "-r${MY_REV}" "${MY_TRUNK_DIR}" > "${MY_TMP_FILE}"; then
             echo "error: failed to get log entry for revision ${MY_REV}"
             exit 1;
         fi
@@ -80,6 +113,15 @@ else
             exit 1;
         fi
     done
+
+    # This is a line ending hack for windows hosts.
+    if    "${MY_SED}" -e 's/1/1/g' --output-text "${MY_TMP_FILE}" "${MY_MSG_FILE}" \
+       && "${MY_SED}" -e 's/1/1/g' --output-text "${MY_MSG_FILE}" "${MY_TMP_FILE}"; then
+        :
+    else
+        echo "error: SED failed to clean up commit message line-endings."
+        exit 1;
+    fi
 fi
 "${MY_RM}" -f -- "${MY_TMP_FILE}"
 
@@ -89,6 +131,7 @@ fi
 echo "***"
 echo "*** Commit message:"
 "${MY_CAT}" "${MY_MSG_FILE}"
+echo "*** end commit message ***"
 echo "***"
 IFS=`"${MY_PRINTF}" " \t\r\n"` # windows needs \r for proper 'read' operation.
 for MY_IGNORE in 1 2 3; do
@@ -106,13 +149,14 @@ for MY_IGNORE in 1 2 3; do
                 "${MY_SVN}" up "${MY_BRANCH_DIR}"
                 exit 0
             fi
-            echo "error: commit failed" >2
+            echo "error: commit failed" 1>&2
             exit 1
             ;;
         n|N|[nN][oO])
             exit 1
             ;;
         *)
+            echo
             echo "Please answer 'y' or 'n'... (MY_ANSWER=${MY_ANSWER})"
     esac
 done
